Attribute VB_Name = "SecurityEnforcement"
Attribute VB_Description = "Security proceedures"
Option Explicit
'This module contains the declarations relating to password encryption
'and access permissions

'Permision Variables
Type User
    'user name
    UserName As String
    
    'Classes group
    Allowed_Classes_CreateNew As Boolean
    Allowed_Classes_Edit As Boolean
    Allowed_Classes_Delete As Boolean
    Allowed_Classes_BatchUpdate As Boolean
    
    'Students Group
    Allowed_Students_Register As Boolean
    Allowed_Students_Edit As Boolean
    Allowed_Students_Deregister As Boolean
    Allowed_Students_Reregister As Boolean
    
    'Fee Group
    Allowed_Fees_Registered_AddNew As Boolean
    Allowed_Fees_Registered_Delete As Boolean
    Allowed_Fees_Templates_CreateNewAndModify As Boolean
    Allowed_Fees_Templates_Delete As Boolean
    
    'Levy Fees Account Group
    Allowed_LevyAc_Debit As Boolean
    Allowed_LevyAc_Credit As Boolean
    Allowed_LevyAc_IndividualStmt As Boolean
    Allowed_LevyAc_BatchStmt As Boolean
    
    'Deposit Account Group
    Allowed_DepositAc_Debit As Boolean
    Allowed_DepositAc_Credit As Boolean
    Allowed_DepositAc_IndividualStmt As Boolean
    Allowed_DepositAc_BatchStmt As Boolean
    
    'Balance Analysis
    Allowed_Balances_FeeClass As Boolean
    Allowed_Balances_LevyAc As Boolean
    Allowed_Balances_DepositAc As Boolean
    
    'Utilities
    Allowed_MailMerge As Boolean
    Allowed_AdvancedSearch As Boolean
End Type

Public CurrentUser As User

'error to generate if a username is not found
Public Const errUserNotFound = vbObjectError + 600 + 1


Public Sub InitialiseUserPermissions(strUserName As String)
    
    If TextTheSame(strUserName, "admin") Then
        SetAdminRights
    Else
        SetUserRights (strUserName)
    End If
    
    
    
End Sub

Private Sub SetUserRights(strUserName As String)
    
    rstUsers.Index = "User Name Index"
    rstUsers.Seek "=", strUserName
    If rstUsers.NoMatch Then
        MsgBox "User account not found", vbExclamation
    Else
        'Classes group
        CurrentUser.Allowed_Classes_CreateNew = getRight(rstUsers("User Rights"), 1)
        CurrentUser.Allowed_Classes_Edit = getRight(rstUsers("User Rights"), 2)
        CurrentUser.Allowed_Classes_Delete = getRight(rstUsers("User Rights"), 3)
        CurrentUser.Allowed_Classes_BatchUpdate = getRight(rstUsers("User Rights"), 4)
        
        'Students Group
        CurrentUser.Allowed_Students_Register = getRight(rstUsers("User Rights"), 5)
        CurrentUser.Allowed_Students_Edit = getRight(rstUsers("User Rights"), 6)
        CurrentUser.Allowed_Students_Deregister = getRight(rstUsers("User Rights"), 7)
        CurrentUser.Allowed_Students_Reregister = getRight(rstUsers("User Rights"), 8)
        
        'Fee Group
        CurrentUser.Allowed_Fees_Registered_AddNew = getRight(rstUsers("User Rights"), 9)
        CurrentUser.Allowed_Fees_Registered_Delete = getRight(rstUsers("User Rights"), 10)
        CurrentUser.Allowed_Fees_Templates_CreateNewAndModify = getRight(rstUsers("User Rights"), 11)
        CurrentUser.Allowed_Fees_Templates_Delete = getRight(rstUsers("User Rights"), 12)
        
        'Levy Fees Account Group
        CurrentUser.Allowed_LevyAc_Debit = getRight(rstUsers("User Rights"), 13)
        CurrentUser.Allowed_LevyAc_Credit = getRight(rstUsers("User Rights"), 14)
        CurrentUser.Allowed_LevyAc_IndividualStmt = getRight(rstUsers("User Rights"), 15)
        CurrentUser.Allowed_LevyAc_BatchStmt = getRight(rstUsers("User Rights"), 16)
        
        'Deposit Account Group
        CurrentUser.Allowed_DepositAc_Debit = getRight(rstUsers("User Rights"), 17)
        CurrentUser.Allowed_DepositAc_Credit = getRight(rstUsers("User Rights"), 18)
        CurrentUser.Allowed_DepositAc_IndividualStmt = getRight(rstUsers("User Rights"), 19)
        CurrentUser.Allowed_DepositAc_BatchStmt = getRight(rstUsers("User Rights"), 20)
        
        'Balance Analysis
        CurrentUser.Allowed_Balances_FeeClass = getRight(rstUsers("User Rights"), 21)
        CurrentUser.Allowed_Balances_LevyAc = getRight(rstUsers("User Rights"), 22)
        CurrentUser.Allowed_Balances_DepositAc = getRight(rstUsers("User Rights"), 23)
        
        'Mail Merge
        CurrentUser.Allowed_MailMerge = getRight(rstUsers("User Rights"), 24)
        
        'Advanced Search
        CurrentUser.Allowed_AdvancedSearch = getRight(rstUsers("User Rights"), 25)
        
        'set user name
        CurrentUser.UserName = rstUsers("User Name")
    End If
    
End Sub

Private Function getRight(strRights As String, intRight As Integer) As Integer
    
    If Mid(strRights, intRight, 1) = "1" Then
        getRight = True
    Else
        getRight = False
    End If

End Function

Private Sub SetAdminRights()
    
    'Classes group
    CurrentUser.Allowed_Classes_CreateNew = True
    CurrentUser.Allowed_Classes_Edit = True
    CurrentUser.Allowed_Classes_Delete = True
    CurrentUser.Allowed_Classes_BatchUpdate = True
    
    'Students Group
    CurrentUser.Allowed_Students_Register = True
    CurrentUser.Allowed_Students_Edit = True
    CurrentUser.Allowed_Students_Deregister = True
    CurrentUser.Allowed_Students_Reregister = True
    
    'Fee Group
    CurrentUser.Allowed_Fees_Registered_AddNew = True
    CurrentUser.Allowed_Fees_Registered_Delete = True
    CurrentUser.Allowed_Fees_Templates_CreateNewAndModify = True
    CurrentUser.Allowed_Fees_Templates_Delete = True
    
    'Levy Fees Account Group
    CurrentUser.Allowed_LevyAc_Debit = True
    CurrentUser.Allowed_LevyAc_Credit = True
    CurrentUser.Allowed_LevyAc_IndividualStmt = True
    CurrentUser.Allowed_LevyAc_BatchStmt = True
    
    'Deposit Account Group
    CurrentUser.Allowed_DepositAc_Debit = True
    CurrentUser.Allowed_DepositAc_Credit = True
    CurrentUser.Allowed_DepositAc_IndividualStmt = True
    CurrentUser.Allowed_DepositAc_BatchStmt = True
    
    'Balance Analysis
    CurrentUser.Allowed_Balances_FeeClass = True
    CurrentUser.Allowed_Balances_LevyAc = True
    CurrentUser.Allowed_Balances_DepositAc = True
    
    'Mail Merge
    CurrentUser.Allowed_MailMerge = True
    
    'Advanced Search
    CurrentUser.Allowed_AdvancedSearch = True
    
    'set user name
    CurrentUser.UserName = "Admin"
    
End Sub

Public Sub GiveSecViolationMsg(strMsg As String)
    strMsg = "You have not been authorised to " & strMsg & "."
    MsgBox strMsg, vbExclamation, "Access Denied"
    
End Sub

Public Function GetUserPassword(strUserName As String) As String
    'THROWS EXCEPTION errUserNotFound
    
    'first check if admin
    If TextTheSame(strUserName, "admin") Then
        'get password from registry
        GetUserPassword = Decrypt(GetSetting(regAppName, regSection, regKeyAdminPassword))
    Else
        'if not adminstrator then look for account in database
        rstUsers.Index = "User Name Index"
        rstUsers.Seek "=", strUserName
        If rstUsers.NoMatch Then
            'Account not found, raise error
            Err.Raise errUserNotFound, "Get User Password", "User Not Found"
        Else
            'decrypt and return password
            GetUserPassword = Decrypt(rstUsers("User Password"))
        End If
    End If
End Function

Public Sub SetUserPassword(strUserName As String, strUserPassword As String)
    'THROWS EXCEPTION errUserNotFound
    
    If TextTheSame(strUserName, "admin") Then
        'update admin password in registry
        Call SaveSetting(regAppName, regSection, regKeyAdminPassword, Encrypt(strUserPassword))
    Else
        rstUsers.Index = "User Name Index"
        rstUsers.Seek "=", strUserName
        If rstUsers.NoMatch Then
            'Account not found, raise error
            Err.Raise errUserNotFound, "Set User Password", "User Not Found"
        Else
            rstUsers.Edit
                rstUsers("User Password") = Encrypt(strUserPassword)
            rstUsers.Update
        End If
    End If

End Sub

Public Sub ResetCurrentUser()
    
    'User Name
    CurrentUser.UserName = ""
    
    'Classes group
    CurrentUser.Allowed_Classes_CreateNew = False
    CurrentUser.Allowed_Classes_Edit = False
    CurrentUser.Allowed_Classes_Delete = False
    CurrentUser.Allowed_Classes_BatchUpdate = False
    
    'Students Group
    CurrentUser.Allowed_Students_Register = False
    CurrentUser.Allowed_Students_Edit = False
    CurrentUser.Allowed_Students_Deregister = False
    CurrentUser.Allowed_Students_Reregister = False
    
    'Fee Group
    CurrentUser.Allowed_Fees_Registered_AddNew = False
    CurrentUser.Allowed_Fees_Registered_Delete = False
    CurrentUser.Allowed_Fees_Templates_CreateNewAndModify = False
    CurrentUser.Allowed_Fees_Templates_Delete = False
    
    'Levy Fees Account Group
    CurrentUser.Allowed_LevyAc_Debit = False
    CurrentUser.Allowed_LevyAc_Credit = False
    CurrentUser.Allowed_LevyAc_IndividualStmt = False
    CurrentUser.Allowed_LevyAc_BatchStmt = False
    
    'Deposit Account Group
    CurrentUser.Allowed_DepositAc_Debit = False
    CurrentUser.Allowed_DepositAc_Credit = False
    CurrentUser.Allowed_DepositAc_IndividualStmt = False
    CurrentUser.Allowed_DepositAc_BatchStmt = False
    
    'Balance Analysis
    CurrentUser.Allowed_Balances_FeeClass = False
    CurrentUser.Allowed_Balances_LevyAc = False
    CurrentUser.Allowed_Balances_DepositAc = False
    
    'Mail Merge
    CurrentUser.Allowed_MailMerge = False
    
    'Advanced Search
    CurrentUser.Allowed_AdvancedSearch = False
    
End Sub
